from setuptools import setup, find_packages

PKG = "ticketsdk"
version = "2.2.5"
long_desc = (
    """This SDK is a programatic inteface into the ticket APIs of NandH Logistics."""
)

setup(
    name=PKG,
    version=version,
    description="ticket SDK for Python",
    author="it@nandhlogistics.vn",
    author_email="it@nandhlogistics.vn",
    url="https://github.com/ITNHL/nh-ticket-sdk",
    license="Version 1.0",
    packages=find_packages(include=["ticketsdk", "ticketsdk.*"]),
    provides=[PKG],
    test_suite="tests",
    long_description=long_desc,
    install_requires=[
        "pre-commit==3.6.2",
        "marshmallow==3.19.0",
        "requests==2.31.0",
    ],
    dependency_links=["file:dist/ticketsdk-2.2.1.tar.gz"],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
    ],
)



from marshmallow import RAISE, Schema, ValidationError, fields

list_issue_code = ["INBOUND", "OUTBOUND", "RETURN_GOODS", "INVENTORY_CHECKING"]

list_from_system = ["WMS", "OMS"]


def validate_issue_code(val):
    if val not in list_issue_code:
        raise ValidationError(f"issue_code in : {list_issue_code}")


def validate_from_system(val):
    if val not in list_from_system:
        raise ValidationError(f"from_system in : {list_from_system}")


def validate_type_list(val):
    if val not in ["list"]:
        raise ValidationError(f"lambda_type not exists")


def validate_type_detail(val):
    if val not in ["detail"]:
        raise ValidationError(f"lambda_type not exists")


def validate_type_comment(val):
    if val not in ["get_comments", "add_comments"]:
        raise ValidationError(f"lambda_type not exists")


class NewTicketSchema(Schema):
    issue_code = fields.Str(required=True, validate=validate_issue_code)
    from_system = fields.Str(required=True, validate=validate_from_system)
    ref_code = fields.Str(required=True)
    partner_code = fields.Str(required=True)
    ticket_name = fields.Str(required=True)
    requested_email = fields.Str(required=True)
    ticket_description = fields.Str(required=True)
    attachments = fields.List(fields.Str(), allow_none=False, allow_empty=True)
    lambda_type = fields.Str(required=True)

    class Meta:
        strict = True
        unknown = RAISE


class NewSellerSchema(Schema):
    partner_code = fields.Str(required=True)
    email = fields.Str(required=True)
    cs_email = fields.Str(required=True)
    lambda_type = fields.Str(required=True)
    name = fields.Str(required=False)
    company_name = fields.Str(required=False)

    class Meta:
        strict = True
        unknown = RAISE


class ListTicketSchema(Schema):
    lambda_type = fields.Str(required=True, validate=validate_type_list)
    page = fields.Int(required=True)
    code = fields.Str(required=False)
    created_at_from = fields.Float(required=False)
    created_at_to = fields.Float(required=False)
    page = fields.Int(required=False)
    status = fields.Str(required=False)
    partner_code = fields.Str(required=False)

    class Meta:
        strict = True
        unknown = RAISE


class DetailTicketSchema(Schema):
    lambda_type = fields.Str(required=True, validate=validate_type_detail)
    ticket_id = fields.Int(required=True)

    class Meta:
        strict = True
        unknown = RAISE


class DetailCommentTicketSchema(Schema):
    lambda_type = fields.Str(required=True, validate=validate_type_comment)
    ticket_id = fields.Int(required=True)

    class Meta:
        strict = True
        unknown = RAISE


class CreateCommentTicketSchema(Schema):
    lambda_type = fields.Str(required=True, validate=validate_type_comment)
    ticket_id = fields.Int(required=True)
    content = fields.Str(required=True)
    attachments = fields.List(fields.Str(), allow_none=True, allow_empty=True)
    partner_code = fields.Str(required=True)

    class Meta:
        strict = True
        unknown = RAISE



from marshmallow import ValidationError

from ticketsdk.constants import LAMBDA_URI, LAMBDA_ENDPOINT_FOR_TICKET, LAMBDA_AUTHEN
from ticketsdk.client import ServiceHttpClient
from ticketsdk.validators import (
    NewTicketSchema,
    NewSellerSchema,
    ListTicketSchema,
    DetailTicketSchema,
    DetailCommentTicketSchema,
    CreateCommentTicketSchema,
)

headers = {
    "Authorization": LAMBDA_AUTHEN,
    "Content-Type": "application/json",
}


class Connection:
    def __init__(self, **kwargs):
        self.body = kwargs.get("body")

    def add_new_seller(self):
        try:
            NewSellerSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(base_url=LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")

    def add_new_ticket(self):
        try:
            NewTicketSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")

    def list_ticket(self):
        try:
            ListTicketSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")

    def detail_ticket(self):
        try:
            DetailTicketSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")

    def list_comment_ticket(self):
        try:
            DetailCommentTicketSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")

    def create_comment_ticket(self):
        try:
            CreateCommentTicketSchema().load(self.body)
        except ValidationError as err:
            raise ValueError(err.messages)
        response = ServiceHttpClient(LAMBDA_URI).execute(
            method="post",
            uri=LAMBDA_ENDPOINT_FOR_TICKET,
            body=self.body,
            headers=headers,
        )
        return response.get("response").get("result")
